@@ -19,6 +19,7 @@ from botocore.config import Config
 from botocore.configprovider import ConfigValueStore
 from botocore.hooks import HierarchicalEmitter
 from botocore.model import ServiceModel
+from botocore.useragent import UserAgentString
 from tests import mock, unittest
 
 
@@ -26,8 +27,23 @@ class TestCreateClientArgs(unittest.TestCase):
     def setUp(self):
         self.event_emitter = mock.Mock(HierarchicalEmitter)
         self.config_store = ConfigValueStore()
+        user_agent_creator = UserAgentString(
+            platform_name=None,
+            platform_version=None,
+            platform_machine=None,
+            python_version=None,
+            python_implementation=None,
+            execution_env=None,
+            crt_version=None,
+        )
         self.args_create = args.ClientArgsCreator(
-            self.event_emitter, None, None, None, None, self.config_store
+            event_emitter=self.event_emitter,
+            user_agent=None,
+            response_parser_factory=None,
+            loader=None,
+            exceptions_factory=None,
+            config_store=self.config_store,
+            user_agent_creator=user_agent_creator,
         )
         self.service_name = 'ec2'
         self.region = 'us-west-2'
@@ -35,6 +51,7 @@ class TestCreateClientArgs(unittest.TestCase):
         self.service_model = self._get_service_model()
         self.bridge = mock.Mock(ClientEndpointBridge)
         self._set_endpoint_bridge_resolve()
+        self._set_resolver_uses_builtin()
         self.default_socket_options = [
             (socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         ]
@@ -64,6 +81,9 @@ class TestCreateClientArgs(unittest.TestCase):
         ret_val.update(**override_kwargs)
         self.bridge.resolve.return_value = ret_val
 
+    def _set_resolver_uses_builtin(self, uses_builtin=True):
+        self.bridge.resolver_uses_builtin_data.return_value = uses_builtin
+
     def call_get_client_args(self, **override_kwargs):
         call_kwargs = {
             'service_model': self.service_model,
@@ -75,6 +95,12 @@ class TestCreateClientArgs(unittest.TestCase):
             'scoped_config': {},
             'client_config': None,
             'endpoint_bridge': self.bridge,
+            'endpoints_ruleset_data': {
+                'version': '1.0',
+                'parameters': {},
+                'rules': [],
+            },
+            'partition_data': {},
         }
         call_kwargs.update(**override_kwargs)
         return self.args_create.get_client_args(**call_kwargs)
@@ -189,7 +215,7 @@ class TestCreateClientArgs(unittest.TestCase):
         )
         self.assertEqual(client_args['client_config'].region_name, None)
 
-    def test_tcp_keepalive_enabled(self):
+    def test_tcp_keepalive_enabled_scoped_config(self):
         scoped_config = {'tcp_keepalive': 'true'}
         with mock.patch('botocore.args.EndpointCreator') as m:
             self.call_get_client_args(scoped_config=scoped_config)
@@ -200,12 +226,38 @@ class TestCreateClientArgs(unittest.TestCase):
             )
 
     def test_tcp_keepalive_not_specified(self):
-        scoped_config = {}
         with mock.patch('botocore.args.EndpointCreator') as m:
-            self.call_get_client_args(scoped_config=scoped_config)
+            self.call_get_client_args(scoped_config={}, client_config=None)
             self.assert_create_endpoint_call(
                 m, socket_options=self.default_socket_options
             )
+            self.call_get_client_args(
+                scoped_config=None, client_config=Config()
+            )
+            self.assert_create_endpoint_call(
+                m, socket_options=self.default_socket_options
+            )
+
+    def test_tcp_keepalive_enabled_if_set_anywhere(self):
+        with mock.patch('botocore.args.EndpointCreator') as m:
+            self.call_get_client_args(
+                scoped_config={'tcp_keepalive': 'true'},
+                client_config=Config(tcp_keepalive=False),
+            )
+            self.assert_create_endpoint_call(
+                m,
+                socket_options=self.default_socket_options
+                + [(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)],
+            )
+            self.call_get_client_args(
+                scoped_config={'tcp_keepalive': 'false'},
+                client_config=Config(tcp_keepalive=True),
+            )
+            self.assert_create_endpoint_call(
+                m,
+                socket_options=self.default_socket_options
+                + [(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)],
+            )
 
     def test_tcp_keepalive_explicitly_disabled(self):
         scoped_config = {'tcp_keepalive': 'false'}
@@ -457,11 +509,40 @@ class TestCreateClientArgs(unittest.TestCase):
         )['client_config']
         self.assertEqual(config.retries['mode'], 'standard')
 
+    def test_creates_ruleset_resolver_if_given_data(self):
+        with mock.patch('botocore.args.EndpointRulesetResolver') as m:
+            self.call_get_client_args(
+                service_model=self._get_service_model('s3'),
+                endpoints_ruleset_data={
+                    'version': '1.0',
+                    'parameters': {},
+                    'rules': [],
+                },
+            )
+            m.assert_called_once()
+
+    def test_doesnt_create_ruleset_resolver_if_not_given_data(self):
+        with mock.patch('botocore.args.EndpointRulesetResolver') as m:
+            self.call_get_client_args(
+                service_model=self._get_service_model('s3'),
+                endpoints_ruleset_data=None,
+            )
+            m.assert_not_called()
+
 
 class TestEndpointResolverBuiltins(unittest.TestCase):
     def setUp(self):
         event_emitter = mock.Mock(HierarchicalEmitter)
         self.config_store = ConfigValueStore()
+        user_agent_creator = UserAgentString(
+            platform_name=None,
+            platform_version=None,
+            platform_machine=None,
+            python_version=None,
+            python_implementation=None,
+            execution_env=None,
+            crt_version=None,
+        )
         self.args_create = args.ClientArgsCreator(
             event_emitter=event_emitter,
             user_agent=None,
@@ -469,6 +550,7 @@ class TestEndpointResolverBuiltins(unittest.TestCase):
             loader=None,
             exceptions_factory=None,
             config_store=self.config_store,
+            user_agent_creator=user_agent_creator,
         )
         self.bridge = ClientEndpointBridge(
             endpoint_resolver=mock.Mock(),
